import sys

if sys.version_info[:2] >= (3, 8):
    # TODO: Import directly (no need for conditional) when `python_requires = >= 3.8`
    from importlib.metadata import PackageNotFoundError, version  # pragma: no cover
else:
    from importlib_metadata import PackageNotFoundError, version  # pragma: no cover

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = __name__
    __version__ = version(dist_name)
except PackageNotFoundError:  # pragma: no cover
    __version__ = "unknown"
finally:
    del version, PackageNotFoundError

from matplotlib import pyplot as plt


class BaseModel:
    """
    Base class for all models both discrete and continuous
    """

    def __init__(self):
        self.name = None
        self.model_type = None
        self.state_variables = {}
        self.parameters = {}
        self.param_values = {}
        self.traces = {}

    def plot_traces(self, vars: list = []):
        """
        Plots the simulations
        :param vars: variables to plot
        """
        for series, data in self.traces.items():
            if series in self.state_variables:
                plt.plot(self.traces['time'], data, label=series)
        plt.legend(loc=0)
        plt.grid()
        plt.title("{} model".format(self.model_type))

    def parameter_table(self, latex: bool = False):
        if self.parameters:
            tbl = {
                'Parameter': list(self.parameters.keys()),
                'Value': list(self.param_values.values()),
                'Symbol': list(self.parameters.values())
            }

            if latex == True:
                out = r"""\begin[l|c|c]{tabular}
                \hline
                Parameter & Value & Symbol \\
                \hline
                """ + \
                    r"\\".join([f"{p}&{v}&{s}" for p, v, s in zip(tbl['Parameter'], tbl['Value'], tbl['Symbol'])]) + \
                    r"""
                    \hline
                    \end{tabular}"""
            else:
                out = tbl
        else:
            out = {}
        return out



import unittest
import sys
sys.path.append('..')
from epimodels.continuous import models as cm
from epimodels.discrete import models as dm


class MyTestCase(unittest.TestCase):
    def test_latex_pars(self):
        model = cm.SIR()
        model([1000, 1, 0], [0, 500], 1001, {'beta': .2, 'gamma': .1})
        assert model.parameter_table(True).startswith("\\begin[l|c|c]{tabular}")
        assert model.parameter_table(True).endswith("\\end{tabular}")

    def test_latex_pars_discrete(self):
        model = dm.SIR()
        model([1000, 1, 0], [0, 500], 1001, {'beta': .2, 'gamma': .1})
        assert model.parameter_table(True).startswith("\\begin[l|c|c]{tabular}")
        assert model.parameter_table(True).endswith("\\end{tabular}")



if __name__ == '__main__':
    unittest.main()



__author__ = 'fccoelho'

import sys

sys.path.append('..')
from matplotlib import pyplot as P
import numpy as np
from epimodels.continuous import (SIS, SIR, SIR1D, SIRS, SEIR, SEQIAHR, Dengue4Strain)


def test_SIR():
    model = SIR()
    model([1000, 1, 0], [0, 500], 1001, {'beta': .2, 'gamma': .1})
    assert len(model.traces) == 4
    # assert len(model.traces['time']) == 50
    model.plot_traces()
    # P.show()


def test_SIR_with_t_eval():
    model = SIR()
    model([1000, 1, 0], [0, 500], 1001, {'beta': .2, 'gamma': .1}, t_eval=range(0, 500))
    assert len(model.traces['S']) == 500
    # assert len(model.traces['time']) == 50


def test_SIR1D():
    model = SIR1D()
    model([0], [0, 500], 100, {'R0': 1.5, 'gamma': .1, 'S0': 98})
    # assert len(model.traces['R']) == 500
    assert len(model.traces) == 2
    model.plot_traces()
    # P.show()


def test_SIS():
    model = SIS()
    model([1000, 1], [0, 50], 1001, {'beta': 2, 'gamma': .1})
    assert len(model.traces) == 3
    # assert len(model.traces['time']) == 50
    model.plot_traces()
    # P.show()


def test_SIRS():
    model = SIRS()
    model([1000, 1, 0], [0, 50], 1001, {'beta': 5, 'gamma': 1.9, 'xi': 0.05})
    assert len(model.traces) == 4
    # assert len(model.traces['time']) == 50
    model.plot_traces()
    # P.show()


def test_SEIR():
    model = SEIR()
    model([1000, 0, 1, 0], [0, 50], 1001, {'beta': 5, 'gamma': 1.9, 'epsilon': 0.1})
    # print(model.traces)
    assert len(model.traces) == 5  # state variables plus time
    # assert len(model.traces['time']) == 50
    model.plot_traces()
    # P.show()


def test_SEQIAHR():
    model = SEQIAHR()
    model([.99, 0, 1e-6, 0, 0, 0, 0, 0], [0, 300], 1, {'chi': .7, 'phi': .01, 'beta': .5,
                                                       'rho': .05, 'delta': .1, 'gamma': .1,
                                                       'alpha': .33, 'mu': .03,
                                                       'p': .75, 'q': 50, 'r': 40
                                                       })
    # print(model.traces)
    assert len(model.traces) == 9  # state variables plus time
    # assert len(model.traces['time']) == 50
    model.plot_traces()
    # P.show()


def test_Dengue4Strain():
    model = Dengue4Strain()
    inits = [48000, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
             0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    pars = {
        'beta': 200 / (50000 * 52),  # 200 novos casos per capita per year
        'N': 50000, #population size
        'delta': 0.5,  # Cross-immunity protection. 1 means no cross-immunity
        'mu': 1 / (1 * 52),  # natural Birth/Mortality rate
        'sigma': 1 / 1.5,  # recovery rate
        'im': 200 #  imported cases
    }
    model(inits, [0, 208], 50000, pars)
    # model.plot_traces()
    pts = len(model.traces['time'])
    Ia1 = np.zeros(pts) # All infectious for strain 1
    Ia2 = np.zeros(pts) # All infectious for strain 2
    Ia3 = np.zeros(pts) # All infectious for strain 3
    Ia4 = np.zeros(pts) # All infectious for strain 4
    Iall = np.zeros(pts)
    for v,tr in model.traces.items():
        if not v.startswith('I_'):
            continue
        Iall += tr
        if v.endswith('1'):
            Ia1 += tr
        elif v.endswith('2'):
            Ia2 += tr
        elif v.endswith('3'):
            Ia3 += tr
        elif v.endswith('4'):
            Ia4 += tr
    P.plot(model.traces['time'], Ia1, label='Infectious strain 1')
    P.plot(model.traces['time'], Ia2, label='Infectious strain 2')
    P.plot(model.traces['time'], Ia3, label='Infectious strain 3')
    P.plot(model.traces['time'], Ia4, label='Infectious strain 4')
    P.plot(model.traces['time'], Iall, label='All infectious')
    P.grid()
    P.legend(loc=0)


    # P.show()

# def test_SIS_with_cache():
#     model = SIS()
#     model([1000, 1], [0, 50], 1001, {'beta': 2, 'gamma': .1})
#     tr1 = model.traces
#     model([1000, 1], [0, 50], 1001, {'beta': 2, 'gamma': .1})
#     tr2 = model.traces
#     assert (tr1 == tr2)
